use std::{collections::BTreeMap, sync::Arc};

use axum::{
    extract::{Path, State},
    response::IntoResponse,
};
use sqlx::{query, query_as};
use tracing::error;

use crate::{
    extractor::ExtractUserToken,
    model::user::{Role, User},
    state::AppState,
    templates::{Page, PageData},
};

pub async fn get_user(
    State(state): State<Arc<AppState>>,
    Path(user_id): Path<i32>,
) -> impl IntoResponse {
    let user = match query_as!(User, r#"SELECT * FROM users WHERE id = $1"#, user_id)
        .fetch_one(&state.pool)
        .await
    {
        Ok(user) => user,
        Err(why) => {
            error!("Getting user: {}", why);
            return Page::new(
                state
                    .hbs
                    .render::<PageData>("error.hbs", &Default::default()),
            );
        }
    };

    Page::new(
        state
            .hbs
            .render("error.hbs", &PageData { user: Some(user) }),
    )
}

pub async fn get_user_new(
    State(state): State<Arc<AppState>>,
    ExtractUserToken(user): ExtractUserToken,
) -> impl IntoResponse {
    if user.is_some()
        && match user.as_ref().unwrap().role {
            Role::Viewer => false,
            Role::Editor => false,
            Role::Manager => false,
            Role::Admin => true,
        }
    {
        // Admins can create accounts
        // Handlebars expects *some* value when rendering a template, so just pass an empty BTreeMap
        let map = BTreeMap::<&str, &str>::new();
        return Page::new(state.hbs.render("new_user.hbs", &map));
    } else if user.is_some() {
        // No permissions to create account
        return Page::new(
            state
                .hbs
                .render::<PageData>("error.hbs", &Default::default()),
        );
    }
    // Else, let the user create a new account if no accounts exist yet

    let count = match query!(r#"SELECT COUNT(id) FROM users"#)
        .fetch_one(&state.pool)
        .await
    {
        Ok(r) => r.count.unwrap_or(0),
        Err(why) => {
            error!("Getting user count: {}", why);
            return Page::new(
                state
                    .hbs
                    .render::<PageData>("error.hbs", &Default::default()),
            );
        }
    };

    let mut map = BTreeMap::new();
    map.insert("count", count);

    Page::new(state.hbs.render("new_user.hbs", &map))
}
